/*
	atkeyb.c
	
	AT-Keyboard library
	Taken from ATMEL Appication Note AVR313
	
	Modified for use with avrgcc
	
	2/2005 Stefan Seegel dahamm@gmx.net
	
	revised by:
	12/2008 Gregor Richter
*/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "atkeyb.h"
#include "scancodes.h"

#define PINOFPORT(x) (*(&x - 2))
#define DDROFPORT(x) (*(&x - 1))


unsigned char bitcount = 11;// 0 = neg. 1 = pos.
unsigned char kb_buffer[BUFF_SIZE];
unsigned char *inpt, *outpt;
volatile unsigned char atkeyb_buffcnt;

void put_kbbuff(unsigned char c);
void decode(unsigned char data);

struct
{
	unsigned char f0_mode	:1;
	unsigned char e0_mode	:1;
	unsigned char shift 	:1;
	unsigned char num 		:1;
	unsigned char alt_gr 	:1;
	unsigned char ascii		:1;
} flag;


void atkeyb_init(void)
{
	inpt = kb_buffer;// Initialize buffer
	outpt = kb_buffer;
	atkeyb_buffcnt = 0;
	DDROFPORT(KEYB_DATA_PORT)&=~(1<<KEYB_DATA_PIN);
	KEYB_DATA_PORT&=~(1<<KEYB_DATA_PIN);
	MCUCR = 1<<ISC01; // INT0 interrupt on falling edge
	//GIFR|=1<<INTF0; //Clear interrut 0 flag
	GICR|=1<<INT0;
	flag.ascii=1;
}

SIGNAL (SIG_INTERRUPT0)
{
	static unsigned char data;// Holds the received scan code
	if (MCUCR == 0x02) // Routine entered at falling edge
	{
		if (bitcount < 11 && bitcount > 2)// Bit 3 to 10 is data. Parity bit,
		{ // start and stop bits are ignored.
			data = (data >> 1);
			if (PINOFPORT(KEYB_DATA_PORT) & (1<<KEYB_DATA_PIN))
				data = data | 0x80;// Store a '1'
		}
		
		MCUCR = 0x03;// Set interrupt on rising edge
	}
	else
	{ // Routine entered at rising edge
		MCUCR = 0x02;// Set interrupt on falling edge
		if (--bitcount == 0)// All bits received
		{
			decode(data);
			bitcount = 11;
		}
	}
}

void decode(unsigned char data)
{
	unsigned char i, a;
	
	if(!flag.ascii)
	{
		put_kbbuff(data);
		return;
	}
	else if(data==0xE0)	//Special characters
	{
		flag.e0_mode = 1;
		return;
	}
	else if(data==0xE1)
	{
		flag.ascii=0;
		return;
	}
	else if(data==0xF0)	//Taste wird losgelassen
	{
		if(flag.e0_mode)
			flag.e0_mode = 0;
		flag.f0_mode = 1;
		return;	
	}	
	
	if(flag.e0_mode && !flag.f0_mode)
	{
		if(data == 0x11) //ALT-GR
		{
			flag.alt_gr = 1;
			return;
		}
	}
	
	if(flag.f0_mode)
	{
		switch(data)
		{
			case 0x11:	//Alt Gr
				flag.alt_gr = 0;
				flag.e0_mode = 0;
				break;

			case 0x12 :// Left SHIFT
				flag.shift = 0;
				break;

			case 0x59 :// Right SHIFT
				flag.shift = 0;
				break;
			
			case 0x77:
				flag.num = 0; //Num Key
				break;
			
			
		}

		flag.f0_mode = 0;
		
		return;
	}
	else
	{
		switch(data)
		{
			case 0x12 :// Left SHIFT
				flag.shift = 1;
				break;

			case 0x59 :// Right SHIFT
				flag.shift = 1;
				break;
			
			case 0x77:
				flag.num = 1; //Num Key
				break;
			
			default:
				if(flag.alt_gr)
				{
					i=0;
					a=pgm_read_byte(&alt_gr_keys[i][0]);
					while ((a) && (a!=data))
					{
						i++;
						a=pgm_read_byte(&alt_gr_keys[i][0]);
					}

					if (a == data)
					{
						put_kbbuff(pgm_read_byte(&alt_gr_keys[i][1]));//1
					}
				}
				else if(flag.e0_mode)
				{
					i=0;
					a=pgm_read_byte(&special_keys[i][0]);
					while ((a) && (a!=data))
					{
						i++;
						a=pgm_read_byte(&special_keys[i][0]);
					}

					if (a == data)
					{
						put_kbbuff(pgm_read_byte(&special_keys[i][1]));//1
					}
					flag.e0_mode = 0;
				}
				else if(flag.shift)// If shift pressed,
				{
					i=0;
					a=pgm_read_byte(&shifted[i][0]);
					while ((a) && (a!=data))
					{
						i++;
						a=pgm_read_byte(&shifted[i][0]);
					}
					
					if (a == data)
					{
						put_kbbuff(pgm_read_byte(&shifted[i][1]));//1
					}
				}
				else	// If shift not pressed
				{ // do a table look-up
					i=0;
					a=pgm_read_byte(&unshifted[i][0]);
					while ((a) && (a!=data))
					{
						i++;
						a=pgm_read_byte(&unshifted[i][0]);
					}
					
					if (a == data)
					{
						put_kbbuff(pgm_read_byte(&unshifted[i][1]));
					}
				}
				
		}
	}
	
	
}

void put_kbbuff(unsigned char c)
{
	if (atkeyb_buffcnt<BUFF_SIZE)// If buffer not full
	{
		*inpt = c;// Put character into buffer
		inpt++; // Increment pointer
		atkeyb_buffcnt++;
		if (inpt >= kb_buffer + BUFF_SIZE)// Pointer wrapping
		inpt = kb_buffer;
	}
}

char atkeyb_getchar(void)
{
	unsigned char byte;
	while(atkeyb_buffcnt == 0);// Wait for data
	
	byte = *outpt;// Get byte
	outpt++; // Increment pointer
	if (outpt >= kb_buffer + BUFF_SIZE)// Pointer wrapping
		outpt = kb_buffer;
	atkeyb_buffcnt--; // Decrement buffer count
	return byte;
}

void atkeyb_Send(uint8_t byte)
{
	uint8_t bitc = 8;
	uint8_t parityc = 0;

	//Interrupt aus
	GICR &= ~(1<<INT0);
	
	//clock fr 100ms auf low
	PORTD &= ~(1<<KEYB_CLOCK_PIN);	
	DDRD |= (1<<KEYB_CLOCK_PIN);
	_delay_us(100); //Sowas bldes^^

	//  Data  low
	PORTD &= ~(1<<KEYB_DATA_PIN);
	DDRD |= (1<<KEYB_DATA_PIN);
	
	// start Clock
	DDRD &= ~(1<<KEYB_CLOCK_PIN);
	PORTD |= (1<<KEYB_CLOCK_PIN);

	while(bitc--)
	{
		while(PIND & (1<<KEYB_CLOCK_PIN));

		if(byte & 0x01)
		{
		  parityc++;
		  PORTD |= (1<<KEYB_DATA_PIN);
		}
		else
		{
		  PORTD &= ~(1<<KEYB_DATA_PIN);
		}

		while(!(PIND & (1<<KEYB_CLOCK_PIN)));

		byte >>= 1;  // next bit
	}

	while(PIND & (1<<KEYB_CLOCK_PIN));


	if(!(parityc & 0x01)){
		PORTD |= (1<<KEYB_DATA_PIN);
	}
	else{
		PORTD &= ~(1<<KEYB_DATA_PIN);
	}

	while(!(PIND & (1<<KEYB_CLOCK_PIN)));
	while(PIND & (1<<KEYB_CLOCK_PIN));


	DDRD &= ~(1<<KEYB_DATA_PIN);   // in
	PORTD |= (1<<KEYB_DATA_PIN);  // pullup

	while(PIND & (1<<KEYB_DATA_PIN));
	while(PIND & (1<<KEYB_CLOCK_PIN));
	
	while(!(PIND & (1<<KEYB_CLOCK_PIN)));
	while(!(PIND & (1<<KEYB_DATA_PIN)));

	GICR |= (1<<INT0);
	
	_delay_us(1200);

}

void atkeyb_set_led(uint8_t byte)
{
	if(byte>7)
		return;
	atkeyb_Send(0xED);
	atkeyb_Send(byte);
}


